/*
*
* Copyright (c) 2004 Sun Microsystems, Inc.
* All rights reserved.
* You may not modify, use, reproduce, or distribute this software except in compliance with the terms
* of the License at: http://developer.sun.com/berkeley_license.html
* Author : Sameer Tyagi, Sun Microsystems s.t@sun.com
* "Patterns and Strategies for Building Document-Based Web Services"
*  http://java.sun.com/developer/technicalArticles/xml/jaxrpcpatterns
*/
import com.examples.soapelement.*;
import org.w3c.dom.Document;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.soap.*;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.File;
import java.io.IOException;

/**
 * Client to demonstrate the use of the anyType and SOAPElement in the WSDL
 */
public class WSDLClient {

    public static void main(String[] args) throws Exception {
        String url = "http://localhost:9090/anysoapelement/jaxrpc";
        if (args.length == 1) {
            url = args[0];
        }
        POService_Impl serviceproxy = new POService_Impl();
        IPurchaseOrder_Stub stub = (IPurchaseOrder_Stub) (serviceproxy.getIPurchaseOrderPort());
        stub._setProperty(javax.xml.rpc.Stub.ENDPOINT_ADDRESS_PROPERTY, url);
        BusinessDocumentReply reply = sendPurchaseOrder(stub);
        SOAPElement payload = reply.get_any();
        printNodeToConsole(payload);
    }

    /**
     * Sends a purchaseorder to the service
     *
     * @param stub
     * @throws java.rmi.RemoteException
     */
    private static BusinessDocumentReply sendPurchaseOrder(IPurchaseOrder_Stub stub) throws Exception {
        BusinessDocumentRequest request = new BusinessDocumentRequest();
        SOAPElement requestdata = createSOAPMessage("purchaseorder.xml");
        request.set_any(requestdata);
        BusinessDocumentReply reply = stub.acceptPO(request);
        return reply;
    }

    /**
     * Method to create a SOAPElement from a file
     *
     * @return SOAPElement
     * @throws Exception
     */
    public static SOAPElement createSOAPMessage(String filename) throws Exception {
        Document doc = readFileCreateDocument(filename);
        if (doc == null)
            throw new IOException("Problem reading cand creating DOM from file,readFileCreateDocument() returned null ");

        SOAPElement parent = SOAPFactory.newInstance().createElement("dummy");
        TransformerFactory factory = TransformerFactory.newInstance();
        Transformer transformer = factory.newTransformer();
        transformer.transform(new DOMSource(doc), new DOMResult(parent));
        return (SOAPElement) parent.getChildElements().next();
    }


    /**
     * Read a file and create a DOM out of it
     *
     * @param filename the file where the XML is stored
     * @return
     */
    private static Document readFileCreateDocument(String filename) {
        Document xml = null;
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setValidating(false);
            factory.setNamespaceAware(true);
// Use the factory to create a DOM parser
            DocumentBuilder parser = factory.newDocumentBuilder();
            xml = parser.parse(new File(filename));

        } catch (Exception e) {
            System.out.println("Couldnt read file " + filename + " " + e);
        }
        return xml;
    }

    public static void printNodeToConsole(org.w3c.dom.Node n) {
        try {
            System.out.println("\n=============================XML Received from Server =========================================");
            TransformerFactory factory = TransformerFactory.newInstance();
            Transformer transformer = factory.newTransformer();
            // also print to console
            transformer.transform(new DOMSource(n), new StreamResult(System.out));
            System.out.println("\n======================================================================");
        } catch (Exception e) {
            System.out.println(e);
        }
    }
}

