/*
*
* Copyright (c) 2004 Sun Microsystems, Inc.
* All rights reserved.
* You may not modify, use, reproduce, or distribute this software except in compliance with the terms
* of the License at: http://developer.sun.com/berkeley_license.html
* Author : Sameer Tyagi, Sun Microsystems s.t@sun.com
* "Patterns and Strategies for Building Document-Based Web Services"
*  http://java.sun.com/developer/technicalArticles/xml/jaxrpcpatterns
*/
package com.examples.soapelement;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.rpc.server.ServiceLifecycle;
import javax.xml.rpc.server.ServletEndpointContext;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.IOException;
import java.io.InputStream;

public class PurchaseOrderService implements IPurchaseOrder, ServiceLifecycle {
    private ServletEndpointContext context;

    public void init(Object context) {
        this.context = (ServletEndpointContext) context;
    }

    public void destroy() {

    }

    public BusinessDocumentReply acceptPO(BusinessDocumentRequest request) throws BusinessDocumentFault, java.rmi.RemoteException {
        if (request == null) {
            throw new BusinessDocumentFault(createExceptionXML("Web Service was passed a null Purchase order"));
        }

        try {
            SOAPElement requestdata = request.get_any();
            // print the reuqest to the console
            printNodeToConsole(requestdata);
            SOAPElement replydata = createSOAPMessage("/Response.xml");
            BusinessDocumentReply reply = new BusinessDocumentReply();
            reply.set_any(replydata);
            return reply;
        } catch (Exception e) {
            throw new BusinessDocumentFault(createExceptionXML("Web Service could not construct a reply :" + e));
        }

    }

    public static SOAPElement createExceptionXML(String msg) {
        try {
            SOAPFactory sf = SOAPFactory.newInstance();
            SOAPElement se = sf.createElement(msg);
            se.addTextNode(msg);
            return se;
        } catch (Exception se) {
            // should never be called
            System.out.println("Exception creating Exception !!" + se);
            return null;
        }

    }

    /**
     * Method to create a SOAPElement from a file
     *
     * @return SOAPElement
     * @throws Exception
     */
    public SOAPElement createSOAPMessage(String filename) throws Exception {
        Document doc = readFileCreateDocument(filename);
        if (doc == null)
            throw new IOException("Problem reading cand creating DOM from file,readFileCreateDocument() returned null ");

        SOAPElement parent = SOAPFactory.newInstance().createElement("dummy");
        TransformerFactory factory = TransformerFactory.newInstance();
        Transformer transformer = factory.newTransformer();
        transformer.transform(new DOMSource(doc), new DOMResult(parent));
        return (SOAPElement) parent.getChildElements().next();
    }


    /**
     * Read a file and create a DOM out of it
     *
     * @param filename the file where the XML is stored
     * @return
     */
    private Document readFileCreateDocument(String filename) throws Exception {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setValidating(false);
        factory.setNamespaceAware(true);
// Use the factory to create a DOM parser
        DocumentBuilder parser = factory.newDocumentBuilder();
// Create a new handler to handle content
//            parser.setErrorHandler(new MyErrorHandler());
        InputStream in = context.getServletContext().getResourceAsStream(filename);

        Document xml = parser.parse(in);
        in.close();

        return xml;
    }

    public static void printNodeToConsole(Node n) {
        try {
            System.out.println("\n=============================XML Received from Server =========================================");
            TransformerFactory factory = TransformerFactory.newInstance();
            Transformer transformer = factory.newTransformer();
            // also print to console
            transformer.transform(new DOMSource(n), new StreamResult(System.out));
            System.out.println("\n======================================================================");
        } catch (Exception e) {
            System.out.println(e);
        }
    }
}
